#include <runtime/lib.h>
#include <buf/buf.h>
#include <buf/buf-queue.h>

buf_t *
buf_queue_get (buf_queue_t *q)
{
	buf_t *p;

	if (q->count == 0) {
		/*debug_printf (CONST("buf_queue_get: returned 0\n"));*/
		return 0;
	}
	assert (q->head >= q->queue);
	assert (q->head < q->queue + q->size);
	assert (*q->head != 0);

	/* Get the first packet from queue. */
	p = *q->head;

	/* Advance head pointer. */
	++q->head;
	--q->count;
	if (q->head >= q->queue + q->size)
		q->head = q->queue;

	/*debug_printf (CONST("buf_queue_get: returned 0x%04x\n"), p);*/
	return p;
}

unsigned char
buf_queue_is_full (buf_queue_t *q)
{
	/*debug_printf (CONST("buf_queue_is_full: returned %d\n"), (q->count == q->size));*/
	return (q->count == q->size);
}

void
buf_queue_put (buf_queue_t *q, buf_t *p)
{
	buf_t **tail;

	/*debug_printf (CONST("buf_queue_put: p = 0x%04x, count = %d, head = 0x%04x\n"), p, q->count, q->head);*/

	/* Must be called ONLY when queue is not full. */
	assert (q->count < q->size);

	/* Compute the last place in the queue. */
	tail = q->head + q->count;
	if (tail >= q->queue + q->size)
		tail -= q->size;

	/* Put the packet in. */
	*tail = p;
	++q->count;
	/*debug_printf (CONST("    on return count = %d, head = 0x%04x\n"), q->count, q->head);*/
}

void
buf_queue_clean (buf_queue_t *q)
{
	for (; q->count > 0; --q->count) {
		buf_free (*q->head);

		/* Advance head pointer. */
		++q->head;
		if (q->head >= q->queue + q->size)
			q->head = q->queue;
	}
}

void
buf_queue_init (buf_queue_t *q, buf_t **buf, int bytes)
{
	q->queue = buf;
	q->size = bytes / sizeof (buf_t*);
	q->head = q->queue;
	q->count = 0;
}
