#include <runtime/lib.h>
#include <kernel/uos.h>
#include <stream/stream.h>

#ifndef NDEBUG
#include <kernel/internal.h>

void
__assert_fail (const char *cond, const char *file, unsigned int line,
	const char *func)
{
	task_t *t;
	lock_t *m;

	debug_printf (CONST("Assertion failed in function `%S':\n%S:%u: %S\n"),
		func, file, line, cond);

	t = task_active.head;
	if (t)
		debug_printf (CONST("\nTask\tAddress\t\tPrio\tStack pointer\tMsg\tFin\n"));
	for (t=task_active.head; t; t=t->next) {
		debug_printf (CONST("%s\t0x%08x\t"), t->name, t);
		debug_printf (t == task_current ? CONST("*%d*") :
			CONST("%d"), t->prio);
		debug_printf (CONST("\t0x%08x\t%p\n"),
			t->stack_context->__jmpbuf[4], t->message);
		if (t->lock)
			debug_printf (CONST("\tLocked by %p\n"), t->lock);
		if (t->wait)
			debug_printf (CONST("\tWaiting for %p\n"), t->lock);
		if (t->slaves.head) {
			debug_printf (CONST("\tOwning:"));
			for (m=t->slaves.head; m; m=m->next)
				debug_printf (CONST(" %p"), m);
			debug_printf (CONST("\n"));
		}
	}
	if (task_active.head)
		debug_printf (CONST("\n"));
	for (;;)
		abort ();
}
#endif
