#include <runtime/lib.h>
#include <kernel/uos.h>
#include <mem/mem.h>
#include <snmp/asn.h>

void
asn_print_type (unsigned char type)
{
	switch (type) {
	case ASN_INTEGER:          debug_printf (CONST("Integer"));          break;
	case ASN_STRING:           debug_printf (CONST("String"));           break;
	case ASN_NULL:             debug_printf (CONST("Null"));             break;
	case ASN_NOSUCHOBJECT:     debug_printf (CONST("NoSuchObject"));     break;
	case ASN_NOSUCHINSTANCE:   debug_printf (CONST("NoSuchInstance"));   break;
	case ASN_ENDOFMIBVIEW:     debug_printf (CONST("EndOfMibView"));     break;
	case ASN_OID:              debug_printf (CONST("Oid"));              break;
	case ASN_IP_ADDRESS:       debug_printf (CONST("IpAddress"));        break;
	case ASN_COUNTER:          debug_printf (CONST("Counter"));          break;
	case ASN_GAUGE:            debug_printf (CONST("Gauge"));            break;
	case ASN_TIME_TICKS:       debug_printf (CONST("TimeTicks"));        break;
	case ASN_OPAQUE:           debug_printf (CONST("Opaque"));           break;
	case ASN_SEQUENCE:         debug_printf (CONST("Sequence"));         break;
	case ASN_GET_REQUEST:      debug_printf (CONST("Get-Request"));      break;
	case ASN_GET_NEXT_REQUEST: debug_printf (CONST("Get-Next-Request")); break;
	case ASN_GET_RESPONSE:     debug_printf (CONST("Response"));         break;
	case ASN_SET_REQUEST:      debug_printf (CONST("Set-Request"));      break;
	case ASN_TRAP:             debug_printf (CONST("Trap"));             break;
	default:                   debug_printf (CONST("[0x%x]"), type);     break;
	}
}

void
asn_print (asn_t *b, unsigned short off)
{
	unsigned short i, n;
	unsigned long ul;

	switch (b->type) {
	default:
		asn_print_type (b->type);
		if (! (b->type & ASN_CONSTRUCTED))
			break;
		for (i=0; i<b->seq.count; ++i) {
			debug_putchar (0, '\n');
			for (n=0; n<off; ++n)
				debug_printf (CONST("    "));
			asn_print (b->seq.arr[i], off+1);
		}
		break;

	case ASN_NULL:
		debug_printf (CONST("(null)"));
		break;

	case ASN_NOSUCHOBJECT:
		debug_printf (CONST("(noSuchObject)"));
		break;

	case ASN_NOSUCHINSTANCE:
		debug_printf (CONST("(noSuchInstance)"));
		break;

	case ASN_ENDOFMIBVIEW:
		debug_printf (CONST("(endOfMibView)"));
		break;

	case ASN_INTEGER:
		debug_printf (CONST("%ld"), b->int32.val);
		break;

	case ASN_COUNTER:
		debug_printf (CONST("c'%lu"), b->uint32.val);
		break;

	case ASN_GAUGE:
		debug_printf (CONST("g'%lu"), b->uint32.val);
		break;

	case ASN_TIME_TICKS:
		debug_printf (CONST("t'%lu"), b->uint32.val);
		break;

	case ASN_IP_ADDRESS:
		ul = b->uint32.val;
		debug_printf (CONST("%d.%d.%d.%d"),
			(unsigned char) (ul >> 24), (unsigned char) (ul >> 16),
			(unsigned char) (ul >> 8), (unsigned char) ul);
		break;

	case ASN_OPAQUE:
		debug_printf (CONST("<"));
		if (b->string.len > 0)
			debug_printf (CONST("%x"), b->string.str[0]);
		else
			debug_printf (CONST("null-opaque"));
		for (i=1; i<b->string.len; ++i)
			debug_printf (CONST("-%x"), b->string.str[i]);
		debug_printf (CONST(">"));
		break;

	case ASN_STRING:
		if (b->string.len <= 0) {
			debug_printf (CONST("null-string"));
			break;
		}
		debug_printf (CONST("\""));
		for (i=0; i<b->string.len; ++i)
			debug_printf (CONST("%c"), b->string.str[i]);
		debug_printf (CONST("\""));
		break;

	case ASN_OID:
		if (b->string.len <= 0)
			debug_printf (CONST("null-oid"));
		for (i=0; i<b->oid.len; ++i)
			debug_printf (CONST(".%d"), b->oid.id[i]);
		break;
	}
}
