/*
 *  .
 *  ,    :
 *  ,  ,     .
 */
typedef struct _stream_t {
	struct _stream_interface_t *interface;
} stream_t;

typedef struct _stream_interface_t {
	void (*putc) (stream_t *u, short c);
	unsigned short (*getc) (stream_t *u);
	int (*peekc) (stream_t *u);
	void (*flush) (stream_t *u);
	unsigned char (*eof) (stream_t *u);
	void (*close) (stream_t *u);
} stream_interface_t;

#define to_stream(x)   ((stream_t*)&(x)->interface)

/*
 *      ,
 * ..     .
 */
#define putchar(x,s)    (x)->interface->putc(to_stream (x), s)
#define getchar(x)	(x)->interface->getc(to_stream (x))
#define peekchar(x)	(x)->interface->peekc(to_stream (x))
#define fflush(x)       if ((x)->interface->flush) \
			(x)->interface->flush(to_stream (x))
#define feof(x)		((x)->interface->eof ? \
			(x)->interface->eof(to_stream (x)) : 0)
#define fclose(x)       if ((x)->interface->close) \
			(x)->interface->close(to_stream (x))

#define puts(x,str)	stream_puts (to_stream (x), str)
#define gets(x,str,n)	stream_gets (to_stream (x), str, n)
#define vprintf(x,f,a)	stream_vprintf (to_stream (x), f, a)
#define vscanf(x,f,a)	stream_vscanf (to_stream (x), f, a)
#ifdef __GNUC__
#define printf(x,f,a...) stream_printf (to_stream (x), f, ##a)
#endif

void stream_puts (stream_t *u, const char *str);
char *stream_gets (stream_t *u, char *str, int len);
int stream_printf (stream_t *u, const char *fmt, ...);
int stream_vprintf (stream_t *u, const char *fmt, va_list args);
int stream_scanf (stream_t *u, const char *fmt, ...);
int stream_vscanf (stream_t *u, const char *fmt, va_list argp);

int snprintf (char *buf, int size, const char *fmt, ...);
int vsnprintf (char *buf, int size, const char *fmt, va_list args);
int sscanf (const char *buf, const char *fmt, ...);

/*
 *      .
 *  snprintf  ..
 */
typedef struct {
	const stream_interface_t *interface;
	char *buf;
	int size;
} stream_buf_t;

stream_t *stropen (stream_buf_t *u, char *buf, int size);
void strclose (stream_buf_t *u);
